!=============================================================!
!======= module to discretize ar(1) stochastic process =======!
!=============================================================!
module discretizear1
    use globals
    implicit none
    contains
    
    subroutine rouwenhurst(rho, mu_eps, sigma_eps, n, zvect, pmat)
        use mathutils
        implicit none
        
        real(dp), intent(in):: rho, mu_eps, sigma_eps
        integer, intent(in):: n
        real(dp), intent(out):: zvect(n)
        real(dp), intent(out):: pmat(n,n)
        
        real(dp) mu_z, sigma_z, q, eps
        real(dp), allocatable, dimension(:,:):: p1, p2
        integer status, i, j       
        
        mu_z = mu_eps/(1-rho)
        sigma_z = sigma_eps/sqrt(1-rho**2)
        
        q = (rho+1)/2
        eps = sqrt(dble(n-1)) * sigma_z
        
        if (n == 1) then
            pmat = 1.0d0
            zvect = mu_z
            return
        else if (n == 2) then
            pmat = reshape((/q, 1-q, 1-q, q/),(/2,2/))  
            zvect = (/mu_z-eps,mu_z+eps/)
            return
        end if
        
        allocate(p1(2,2),stat=status)        
        p1 = reshape((/q, 1-q, 1-q, q/),(/2,2/))        
                        
        do i=2,n-1            
            allocate(p2(i+1,i+1),stat=status)
            p2 = q * reshape( (/  (/(p1(:,j),0.0d0 ,j=1,i)/) ,  (/(0.0d0,j=1,i+1)/)    /), (/i+1,i+1/) ) + &
                 (1-q) * reshape( (/  (/(0.0d0,j=1,i+1)/), (/ (p1(:,j),0.0d0 ,j=1,i)/)   /) ,   (/i+1,i+1/) ) + &
                 (1-q) * reshape( (/  (/ (0.0d0,p1(:,j) ,j=1,i) /) ,  (/(0.0d0,j=1,i+1)/)  /), (/i+1,i+1/) ) + &
                 q * reshape( (/ (/(0.0d0,j=1,i+1)/), (/(0.0d0,p1(:,j) ,j=1,i)/)   /) ,   (/i+1,i+1/) )
            
            p2(2:i,:) = p2(2:i,:)/2
            
            deallocate(p1,stat=status)            
            
            if (i==n-1) then
                pmat = p2
            else
                allocate(p1(i+1,i+1), stat=status)
                p1 = p2
            end if
            
            deallocate(p2,stat=status)
        end do
        
        zvect = linspace(mu_z-eps,mu_z+eps,n)

    end subroutine rouwenhurst

end module discretizear1