!=============================================================!
!======= global contains main parameters and variables =======!
!=============================================================!
module globals

    !======= family type =======!
	integer                 :: famtype          ! 0 for no demographics, household income equivalized ex post
	                                            ! 2 for deterministic family
	                                            	                                            
    !======= utility weight type =======!	                                            
	integer                 :: uwtype           ! 1 for utility weight=independent of family size
	                                            ! 2 for utility weight=equivalence scale
	                                            ! 3 for utility weight=family size

	!======= precision and others =======!
	integer, parameter		:: dp = 8
	real(dp), parameter		:: one = 1.0_dp
	real(dp), parameter		:: two = 2.0_dp
	real(dp), parameter		:: cmin = 1.0e-2
	
	integer  :: calibrate,calid 

	!======= counters galore =======!
	integer					:: i,j,ii,jj
	
	!======= life-span parameters =======!
	integer, parameter		:: t0 = 25			        ! start of economic life
	integer, parameter		:: tt = 95      	        ! death/final period
	integer, parameter		:: tw = 65      	        ! total # of periods working
	integer, parameter		:: tr = tw+1                ! total # of periods retired	
	
	real(dp)                :: surv(t0-1:tt)
	real(dp)                :: alive(t0-1:tt),unconsurv(t0:tt),alivew(t0:tw)

	!======= family size parameters =======!
	integer		            :: nad  			        ! number of adults in hh.
    integer		            :: nch        		        ! max number of children in hh.
    
    real(dp)                :: detadults(t0:tt), &      ! deterministic profile for 
                               detchild(t0:tt)          ! n. of adults and children
                                                        ! in hh.
                               
    real(dp),allocatable,dimension(:) :: fam1   ! pre-multiplication factor for updating vprw
    real(dp),allocatable,dimension(:) :: fam2   ! pre-multiplication factor for cstr                 
	
	!======= preference parameters =======!
	real(dp)				:: eta,beta,r,lambda
	real(dp),allocatable,dimension(:) :: phi,delta  ! equivalence scale, utility weight
	real(dp),allocatable,dimension(:) :: phi_inc    ! income adjustment for famtype=0
	real(dp)                :: atta1,atta2              ! theta1 and theta2 from attanasios et al. (jbes, 1999)
	
	real(dp)                :: kytarget,ctarget,totcons
	
	real(dp)                :: kymodel,c25c75
	
	!======= tax parameters =======!
	real(dp)				:: tau0,tau1,tau2
	
	!======= pensions =======!
	real(dp),allocatable,dimension(:)       :: pens     ! pensions
    integer,dimension(1)    :: pension_id,demo_id,inc_id
    real(dp)                :: eff_pens	                                                    

	!======= wage shock =======!
	integer         		:: nz,nzt,nzf               ! number of permanent (zp), transitory (zt) 
	                                                    ! and fixed effect shocks
	real(dp)				:: rho,sig_z,sig_zt,sig_zf
	    	
	real(dp),allocatable,dimension(:)	:: zgrid,ztgrid,z0grid,zfgrid
	real(dp),allocatable,dimension(:)	:: ztprob,z0prob,zfprob
	real(dp)				:: sig_e0                   ! distribution of first wage shock (sim. stage)
	real(dp),allocatable,dimension(:,:)	:: ztrans
	
	real(dp),allocatable,dimension(:) :: lcp          ! lifecycle profiles
	
	!======= asset grid =======!
	integer, parameter		:: na = 101			        ! grid points in asset space
	integer, parameter		:: nr = 45  		        ! grid points in pension grid
	                                                    ! (also, number of grid points in avg. earning grid)
	real(dp), parameter		:: amax = 2000000.0	        ! max assets
	
	real(dp), parameter		:: expgrid_w = 0.1	        ! to construct exp. grid, working stage
	real(dp), parameter		:: expgrid_a = 0.1	        ! to construct exp. grid on avg. earnings
	real(dp), parameter		:: expgrid_r = 0.0	        ! to construct exp. grid, retirement
	real(dp)                :: avgearn(t0:tw,nr)        ! avg earnings grid
	real(dp)                :: pgrid(nr)                ! pension grid
	real(dp),allocatable,dimension(:,:,:) :: agrid, &       ! initial asset grids
	                                         agridr
	real(dp),allocatable,dimension(:,:) :: aminr           ! borrowing limits during retirement	                    
	real(dp),allocatable,dimension(:,:) :: amin           ! borrowing limits during working age	                    
	real(dp),allocatable,dimension(:,:) :: amaxr
	real(dp),allocatable,dimension(:,:) :: amaxw
	real(dp)                            :: agridr0(na) ! endogenous asset grids retirement stage
	real(dp)                            :: agrid0(na)  ! endogenous asset grids working stage	
	                           

	!======= value and policy functions =======!
	
	integer,allocatable :: pind(:,:,:,:)  ! to allocate shocks in solution stage 
	                                      ! (to avoid the limit in fortran of 
	                                      !   7 dimensions per array)
	                                      ! dimensions are: nr,nz,nzt,nzf
	
	!----- retirement -----!
	real(dp),allocatable,dimension(:,:,:) :: cstrr, &          ! optimal cons (c 'star') in retirement
	                                         ducstrr
	real(dp)                              :: yendr(na)         ! cash at hand, retirement
	
	!----- working -----!	
	real(dp),allocatable,dimension(:,:,:) :: cstrw, &         ! optimal cons (c 'star') when working
	                                         ducstrw
	real(dp)                              :: yendw(na)        ! cash at hand, work                               
	
	!======= for simulation stage =======!
	integer                             :: nsim 
	real(dp),allocatable,dimension(:,:) :: esim,zsim            ! to store wage shocks
	real(dp),allocatable,dimension(:,:) :: etsim,ztsim
	real(dp),allocatable,dimension(:)   :: efsim,zfsim
	                                    
	real(dp),allocatable,dimension(:,:) :: wsim,wsimout,wsim_eq    ! to store simulated gross earnings
	real(dp),allocatable,dimension(:,:) :: avesim,avesim1
	real(dp),allocatable,dimension(:)   :: pensim
	
	real(dp),dimension(t0:tt)   :: lcmde,lcmean,cmean,cmean_eq,cmean_eq_de
	
	! period average simulated earnings by number of adults 
	real(dp),allocatable,dimension(:) :: mwsim,mwsim_eq                    
	! life-time average simulated earnings by number of adults
	real(dp)                          :: cswsim,cswsim_eq                    
    real(dp),dimension(tw)  :: minwsim,maxwsim,medwsim  ! min,max,median period average average earnings
	                           
    integer,allocatable,dimension(:,:)                 :: izloc,iztloc,aveloc
    integer,allocatable,dimension(:)                   :: izfloc
    
    real(dp),allocatable,dimension(:,:)                :: asim,csim,csim_eq
	                           
	real(dp),allocatable,dimension(:,:)                :: psim            ! to store simulated pensions
	
	!======= extras =======!
	real(dp),allocatable    :: tempr0(:),tempr1(:)
	integer,allocatable     :: tempi0(:),tempi1(:)
	
	!======= filenames =======!	
    character(len=3) ::  fn0,fn1,fn2,fn3,fn4,fn5
    character(len=100) :: fname,fname_mean,fname_ltc,fname_de,fname_bm	
		
end module globals
