!================================================================!
!======= module with family part outside the optimization =======!
!================================================================!
module mod_grids

use globals
use mod_policy
implicit none	

contains

	subroutine grids
	! creates grids for assets during working age
	! and assets during retirement. 
	! takes into account the discretized wage shocks
	    use mathutils
	    integer     :: it,ia,pim,itaux,izf
	    real(dp)    :: minw(t0:tw,nzf),minwaux(t0:tw),maxw(t0:tw),meanw(t0:tw),minpen(tr:tt)
	    real(dp)    :: avminw,avmaxw,avw
	    real(dp)    :: scale
               
        ! grid for average earnings during lifecycle

        pim = nint(one*nr*0.5)
        
        do it = t0,tw
            avgearn(it,1:pim)  = linspace(minwsim(it),medwsim(it),pim)
            avgearn(it,pim:nr) = linspace(medwsim(it),maxwsim(it),pim)
        enddo

        do j = 1,nr
            pgrid(j) = pension(avgearn(tw,j))
        enddo
        
        ! create borrowing limits during retirement
        do j = 1,nr
            aminr(tt,j) = -( pgrid(j)/phi_inc(tt) )/(r/surv(tt-1))
        enddo

        do it = tt-1,tr,-1
            do j = 1,nr
                aminr(it,j) = aminr(it+1,j)/(r/surv(it-1)) - ( pgrid(j)/phi_inc(it) )/(r/surv(it-1))
            enddo
        enddo
        
        ! borrowing limits when working
        do izf = 1,nzf
            minw(:,izf) = exp( lcp(:) &
                             + zfgrid(izf) &
                             + ztgrid(1)   &
                             + zgrid(1) )
                             
            minw(t0,izf) = exp( lcp(t0) &
                             + zfgrid(izf) &
                             + ztgrid(1)   &
                             + zgrid(nint(nz/two)) )

            ! only borrow against future net income
            do it = t0,tw
                minw(it,izf) = minw(it,izf) - policy_tax(minw(it,izf))
            enddo

            ! put in per-adult equivalent terms 
            minw(:,izf) = minw(:,izf)/phi_inc(:)
                        
            ! life-cycle borrowing constraints
            amin(tw,izf) = aminr(tr,1)/(r/surv(tw-1)) - minw(tw,izf)/(r/surv(tw-1))
            
            do it = tw-1,t0,-1
                amin(it,izf) = amin(it+1,izf)/(r/surv(it-1)) - minw(it,izf)/(r/surv(it-1))
            enddo

        enddo
        
        !===============!
        ! max grid size !
        !===============!

        amaxw = amax
        amaxr = amax

        !==========================!
	    ! create exponential grids !
	    !==========================!
	    do izf = 1,nzf
	        do it =t0,tw
	            agrid(it,izf,1) = amin(it,izf)
	            do ia = 2,na
	                agrid(it,izf,ia) = agrid(it,izf,ia-1) + exp( expgrid_w*(ia-1) )
	            enddo
	            scale = (amaxw(it,izf)-agrid(it,izf,1)) / (agrid(it,izf,na)-agrid(it,izf,1))
	            agrid(it,izf,:) = agrid(it,izf,1) + scale*(agrid(it,izf,:)-agrid(it,izf,1))    	        
	        enddo
        enddo
                
	    do it = tr,tt
	        do j = 1,nr
	            agridr(it,j,1) = aminr(it,j)
	            do ia = 2,na
	                agridr(it,j,ia) = agridr(it,j,ia-1) + exp( expgrid_r*(ia-1) )
	            enddo
	            scale = (amaxr(it,j)-agridr(it,j,1)) / (agridr(it,j,na)-agridr(it,j,1))
	            agridr(it,j,:) = agridr(it,j,1) + scale*(agridr(it,j,:)-agridr(it,j,1))
	        enddo	                
	    enddo
	        

	end subroutine grids
	
end module mod_grids   