program main

    !============================!
    !======= dependencies =======!
    !============================!
	use globals
	use mod_user_configurations
	use mod_allocation
	use mod_parametervals	
	use mod_readextdata
	use mod_family
    use mod_zprocess
    use mod_policy
    use mod_grids    
	use mod_solveit	
	use mod_simulate
	use mod_outdata
	use mod_kymodel_calc

	implicit none
	
	real(dp) :: avgf(t0:tt)
	real(dp) :: fbeta,fbeta0,loctol
	real(dp) :: dx,bet_l,bet_u,qr,ql,fraux
	integer  :: specifier,spec_io, itt
    real(dp)           :: zbrnt,faux	
    real(dp),external  :: funBetaWLTH

    dx = 0.02d0
    loctol = 0.00001d0

    print*, " "
	print*, "======= begin computation ======="
	print*, " "

    !--------------------------------------------------------------------------------------------------
	! set some values for specific model
    call setvals    ! (see user_configurations.f90)
    !--------------------------------------------------------------------------------------------------
    
    !--------------------------------------------------------------------------------------------------
	! allocate variables
	call allocation ! (see allocation.f90)
    !--------------------------------------------------------------------------------------------------

    !--------------------------------------------------------------------------------------------------		
	! preference parameters & shock process
	call parametervals  ! (see parametervals.f90)     
    !--------------------------------------------------------------------------------------------------	

    !--------------------------------------------------------------------------------------------------	
	! read deterministic life-cycle profile of wages 
	call readextdata_lcp    ! (see readextdata.f90)

	! read deterministic life-cycle profile of household size and composition
	call readextdata_detfam ! (see readextdata.f90)    	    

    !--------------------------------------------------------------------------------------------------

    !--------------------------------------------------------------------------------------------------	   
	! family part of the model (still outside the optimization)
	
	! equivalence scale, utility weights, and family-related pre-multiplication
    ! factors for euler equation in the endogenous grid-method
	call family ! (see family.f90)
    !--------------------------------------------------------------------------------------------------	   

    !--------------------------------------------------------------------------------------------------	   
    ! stochastic income part of the model
	call zprocess           ! (see zprocess.f90)
	call simulate_earnings  ! (see simulate.f90)
    !--------------------------------------------------------------------------------------------------	   
    ! calculation of the tax rate which funds the soc. security system
    call get_tau0_lin       ! (see policy.f90)
    !--------------------------------------------------------------------------------------------------

	!--------------------------------------------------------------------------------------------------	
	! construction of asset grids during retirement and working life
	call grids  ! (see grids.f90)
	!--------------------------------------------------------------------------------------------------	
	
	!--------------------------------------------------------------------------------------------------	
    ! solution of the model: calibrate beta to match k/y ratio OR simulation for given beta
    !--------------------------------------------------------------------------------------------------	   
    if (calid==0) then
        beta    = 1.05d0
        kymodel = 10.0d0

        print*, " "
        print*, 'start broad zero finder, ky target...'
        do while (kymodel>=kytarget)
            call solveit            ! (see solveit.f90)
            call simulate_lifecycle ! (see simulate.f90)
            call kymodel_calc       ! (see capital_income_ratio.f90)
            bet_u = beta + dx
            bet_l = beta
            beta = beta - dx
            print*, bet_u,bet_l,kymodel
        enddo
        
        print*, " "    
        print*, 'start fine zero finder, ky target'
        call zbrentbetawlth(bet_l,bet_u,loctol,zbrnt)   ! (see zbrentbetawlth.f90)
    else
        print *, 'Type in a discount factor beta'
        read *, beta
        faux=funBetaWLTH(beta)
    endif 
        
    ! save results
    call outdata    ! (see outdata.f90)
    
	!--------------------------------------------------------------------------------------------------	                
        
    print*, " "
	print*, "======= the end !!! ======="
	print*, " "
	
end