!==============================================================!
!======= module with policy parameters (taxes,pensions) =======!
!==============================================================!
module mod_policy

use globals
implicit none	

contains

	function policy_tax(income)
	! calculates period taxes which get substracted
	! from budget constraint

		real(dp)	:: policy_tax,income

		policy_tax = income*tau0

	end function policy_tax


	function pension(mearn)
	! calculates soc. sec. benefits., where 
	!   - mearn: average lifecycle earnings
	!   - cswsim: crossectional avg. earnings
	! see: storesletten, telmer and yaron (jme 2004) 
	! and guvenen, f. (aer 2007)
		real(dp)	:: pension,mearn,thresh(3),frac(4)
		integer     :: imax
		
		thresh(1)=0.3
		thresh(2)=2.0
		thresh(3)=4.1

		thresh=thresh*cswsim
		
		! Replacement ratios
		frac(1)=0.9
		frac(2)=0.32
		frac(3)=0.15
		frac(4)=1.1
		
		pension = frac(1)*min(mearn,thresh(1))
		do i=2,3
			if(mearn>thresh(i-1)) pension = pension+frac(i)*(min(mearn,thresh(i))-thresh(i-1))
	    enddo
	    
		if(mearn>thresh(3)) pension = frac(4)*cswsim		

        pension = real(pension_id(1))*pension
    
	end function pension

    subroutine get_tau0_lin
    ! given simulated life-cycle wages and pensions
    ! this subroutine calculates the level of linear
    ! income taxes that equilibrates perfectly
    ! the social security system
        integer     :: it,isim
        real(dp)    :: taxrev,pxpnd

        do isim = 1,nsim
            pensim(isim) = pension(avesim(tw,isim))
        enddo

        taxrev = 0.0_dp
        pxpnd  = 0.0_dp
                
        !calculate aggregate tax base
        do it = t0,tw
            taxrev = taxrev + alive(it)*sum(wsim(it,:))
        enddo
        
        !calculate aggregate pension expenditures
        do it = tr,tt
            pxpnd = pxpnd + alive(it)*sum(pensim)
        enddo
        
        tau0 = pxpnd/taxrev       
    
    end subroutine get_tau0_lin

	
end module mod_policy    