!===========================================!
!======= module to configure setup   =======!
!===========================================!
module mod_user_configurations

use globals
implicit none	

contains
  
    subroutine setvals
    ! We are always the square root, natural borrowing constraint and risky income
    ! It remains to choose at each run
    !   - family type: single, demographicCa
    !   - utility weight: one, equivalence scale, N
    
        character(len=3) :: fnf,fnu,fnp,fnr,fncal
        integer :: r_id
    
        !========================================!
        ! These are fixed throughout calculations
        !========================================!
        nz  = 17    ! Number of permanent income shocks (17 for baseline results)
        nzt = 2     ! Number of transitory income shocks (2 for baseline results)
        nzf = 2     ! Number of fixed effect shocks (2 for baseline results)
        nsim= 1d5   ! Number of simulated households (1d5 for baseline results)
        
        !==========================!
        ! These are choices
        !==========================!
        !READ SPECIFICATION FROM FILE
        open(1,file='specfile.txt') 
        read(1,*)   famtype
        read(1,*)   uwtype
        read(1,*)   pension_id
        read(1,*)   r
        close(1) 
        
        ! Interest rate
        r_id=r*100
        r=1d0+r/1d2   
        
        ! Target for net wealth to income ratio (as in STORESLETTEN/TELMER/YARON, 2004; KAPLAN-VIOLANTE,2010 use 2.5)
        kytarget=3.1d0
        
!        famtype = 2     ! 0: deterministic, 2: demographics
!        uwtype  = 1     ! in case of famtype == 2, uwtype choices are
!                        !   - 1: delta = one
!                        !   - 2: delta = equiv. scale
!                        !   - 3: delta = N
!                        
!        pension_id  = 1 ! pension system: yes (1) or no (0)

        print *, 'Calibrate the model (0) or simulate for a pre-specified beta (1)?'
        read *, calid
        if (calid==0) then
            fncal=''
        else
            print *,'Type in a number that identifies this simulation'
            read *, fncal
        endif
                 
        write(fnf,'(i3)') famtype
        write(fnu,'(i3)') uwtype
        write(fnp,'(i3)') pension_id
        write(fnr,'(i3)') r_id
        
        fname           ='results\out_f'//trim(adjustl(fnf))//'u'//trim(adjustl(fnu))//'p'//trim(adjustl(fnp))//'r'//trim(adjustl(fnr))//trim(adjustl(fncal))
        fname_mean      =trim(adjustl(fname))//'_mean.txt'
        fname_ltc       =trim(adjustl(fname))//'_ltc.txt'
        
        print*, 'setup (f,u,p):', famtype,uwtype,pension_id
        
    end subroutine setvals
    
end module mod_user_configurations