!=======================================================!
!======= module to discretizes the ar(1) process =======!
!======= earnings shocks, using rouwenhurst's procedure=!
!=======================================================!
module mod_zprocess

use globals
implicit none	

contains

	subroutine zprocess

		use mathutils
		use discretizear1
	
	    integer,parameter               :: simsize = 100000
	    integer                         :: ia,i1,i2,nia
	    real(dp),dimension((simsize))   :: randz,randz0,testz,testz0
	    real(dp)                        :: meanz,meanz0,vz,vz0
	    real(dp),dimension(nz)          :: statd,statdn,x0  
	    
	    real(dp)                        :: tempz0(nz,nz),tempt(nzt,nzt),tempf(nzf,nzf)
	    real(dp)                        :: x0s(nz), xtss(nz)
	    
	    !-----------------------------------------------------------!
	    ! features of the income process for singles                !
		! and deterministic family income process                   !
		!-----------------------------------------------------------!
        
	    ! discretize the permanent component
		call rouwenhurst(rho,0._dp,sig_z,nz,zgrid,ztrans)
        
        ! initial/ergodig distribution of permanent shocks
		call rouwenhurst(0.0_dp,0._dp,sig_z/sqrt(one-rho**two),nz,z0grid,tempz0)
		do i = 1,nz
		    z0prob(i) = tempz0(i,i)
		enddo
				
		! discretize the transitory component
		call rouwenhurst(0._dp,0._dp,sig_zt,nzt,ztgrid,tempt)
		do i = 1,nzt
		    ztprob(i) = tempt(i,i)
		enddo

		! discretize the fixed effect component
		call rouwenhurst(0._dp,0._dp,sig_zf,nzf,zfgrid,tempf)
		do i = 1,nzf
		    zfprob(i) = tempf(i,i)
		enddo
	
	end subroutine zprocess
	
end module mod_zprocess	